@@ -52,8 +52,8 @@ class NestingTree(TouchNotify,nx.DiGraph):
 		self._successor_slots = {}
 		self.touch()
 
-	def add_edge(self, u, v, **kwarg):
-		if 'implied' not in kwarg:
+	def add_edge(self, u, v, implied=False, **kwarg):
+		if not implied:
 			drops = []
 			for u_,v_,imp_ in self.in_edges(nbunch=[v], data='implied'):
 				if imp_:
@@ -61,7 +61,7 @@ class NestingTree(TouchNotify,nx.DiGraph):
 			for d in drops:
 				self._remove_edge_no_implied(*d)
 		self._clear_caches()
-		return super().add_edge(int(u), int(v), **kwarg)
+		return super().add_edge(int(u), int(v), implied=implied, **kwarg)
 
 	def _remove_edge_no_implied(self, u, v, *arg, **kwarg):
 		result = super().remove_edge(u, v)
@@ -76,6 +76,36 @@ class NestingTree(TouchNotify,nx.DiGraph):
 		return result
 
 	def add_node(self, code, *, children=(), parent=None, parents=None, phi_parameters=None, **kwarg):
+		"""
+		Add a single node `code` and update node attributes.
+
+		Parameters
+		----------
+		code : int
+			Although the generic networkx.DiGraph allows a node
+			to be any hashable Python object except None, Larch
+			assumes that node codes are integers.
+		children : Collection
+			A collection of other node codes that are the children
+			of this new node.  Links will be created from this node
+			to each child.
+		parent : int, optional
+			The parent of this new node. If not given, the root
+			node is assumed to be the parent of this node, and an
+			implied link is created.  This implied link is removed
+			if the node is later made the child of some other node.
+			If the parent is set explicitly, the link is *not*
+			removed later.
+		parents : Collection, optional
+			Set multiple parent up-stream nodes.
+		phi_parameters : Mapping
+			Set phi parameters on graph links connecting to this
+			node, used in network GEV models. The keys of this mapping
+			indicate the node at the other end of the link, and the
+			values are parameter names.
+		kwarg : other keyword arguments, optional
+			Set or change node attributes using key=value.
+		"""
 		if parents is not None and parent is not None:
 			raise TypeError("cannot give both parent and parents arguments")
 		super().add_node(code, **kwarg)
@@ -99,7 +129,7 @@ class NestingTree(TouchNotify,nx.DiGraph):
 					raise ValueError(f"connected node {k} from phi_parameters not found")
 		self._clear_caches()
 
-	def new_node(self, **kwarg):
+	def new_node(self, *, code=None, **kwarg):
 		"""
 		Add a new nesting node to this NestingTree.
 
@@ -122,15 +152,28 @@ class NestingTree(TouchNotify,nx.DiGraph):
 			to some other node.
 		name : str, optional
 			A human-readable name to associate with this nest.
+		code : int, optional
+			Use this code for the new nest. If this code already exists,
+			a ValueError is raised.
 
 		Returns
 		-------
 		int
 			The new code for this nest.
+
+		Raises
+		------
+		ValueError
+			If a new code is given but it already exists in this tree.
 		"""
-		proposed_code = len(self)
-		while proposed_code in self:
-			proposed_code += 1
+		if code is None:
+			proposed_code = len(self)
+			while proposed_code in self:
+				proposed_code += 1
+		else:
+			if code in self:
+				raise ValueError(f'code {code} already exists in this tree')
+			proposed_code = code
 		self.add_node(proposed_code, **kwarg)
 		return proposed_code
 
@@ -312,12 +355,19 @@ class NestingTree(TouchNotify,nx.DiGraph):
 		self._successor_slots = {}
 		self.set_touch_callback(None)
 
-	def __xml__(self, **format):
+	def __xml__(self, use_viz=True, use_dot=True, output='svg', figsize=None, **format):
 		viz = None
 		dot = None
-		try:
-			import pygraphviz as viz
-		except ImportError:
+		if use_viz:
+			try:
+				import pygraphviz as viz
+			except ImportError:
+				if use_dot:
+					try:
+						import pydot as dot
+					except ImportError:
+						pass
+		elif use_dot:
 			try:
 				import pydot as dot
 			except ImportError:
@@ -325,8 +375,16 @@ class NestingTree(TouchNotify,nx.DiGraph):
 
 		if viz is None and dot is None:
 			import warnings
-			warnings.warn("pygraphviz module not installed, unable to draw nesting tree")
-			raise NotImplementedError("pygraphviz module not installed, unable to draw nesting tree")
+			if use_viz and use_dot:
+				msg = "neither pydot nor pygraphviz modules are installed, unable to draw nesting tree"
+			elif use_viz:
+				msg = "pygraphviz module not installed, unable to draw nesting tree"
+			elif use_dot:
+				msg = "pydot module not installed, unable to draw nesting tree"
+			else:
+				msg = "no drawing module used, unable to draw nesting tree"
+			warnings.warn(msg)
+			raise NotImplementedError(msg)
 
 		if viz is not None:
 			existing_format_keys = list(format.keys())
@@ -378,16 +436,19 @@ class NestingTree(TouchNotify,nx.DiGraph):
 				up_nodes.add(i)
 			pyg_imgdata = BytesIO()
 			try:
-				G.draw(pyg_imgdata, format='svg', prog='dot')       # write postscript in k5.ps with neato layout
+				G.draw(pyg_imgdata, format=output, prog='dot')       # write postscript in k5.ps with neato layout
 			except ValueError as err:
 				if 'in path' in str(err):
 					import warnings
 					warnings.warn(str(err)+"; unable to draw nesting tree in report")
 					raise NotImplementedError()
-			import xml.etree.ElementTree as ET
-			ET.register_namespace("","http://www.w3.org/2000/svg")
-			ET.register_namespace("xlink","http://www.w3.org/1999/xlink")
-			return ET.fromstring(pyg_imgdata.getvalue().decode())
+			if output=='svg':
+				import xml.etree.ElementTree as ET
+				ET.register_namespace("","http://www.w3.org/2000/svg")
+				ET.register_namespace("xlink","http://www.w3.org/1999/xlink")
+				return ET.fromstring(pyg_imgdata.getvalue().decode())
+			else:
+				raise NotImplementedError(f"output {output} with use_viz")
 		else:
 
 			pydot = dot
@@ -475,13 +536,47 @@ class NestingTree(TouchNotify,nx.DiGraph):
 
 			###
 			from xmle import Elem
-			return Elem.from_any(P.create_svg())
+			prog = None
+			if output == 'svg':
+				import xml.etree.ElementTree as ET
+				ET.register_namespace("","http://www.w3.org/2000/svg")
+				ET.register_namespace("xlink","http://www.w3.org/1999/xlink")
+			elif output == 'png':
+				prog = [P.prog, '-Gdpi=300']
+				if figsize is not None:
+					prog.append(f"-Gsize={figsize[0]},{figsize[1]}\!")
+				e = Elem.from_any(P.create(prog=prog, format=output, **format))
+				e.attrib['dpi'] = (300,300)
+				return e
+			return Elem.from_any(P.create(prog=prog, format=output, **format))
 
 	def _repr_html_(self):
 		from xmle import Elem
 		x = Elem('div') << (self.__xml__())
 		return x.tostring()
 
+	def to_png(self, figsize=None, filename=None):
+		"""
+		Output the graph visualization as a png.
+
+		Parameters
+		----------
+		figsize : 2-tuple, optional
+			The (width, height) in inches.
+
+		Returns
+		-------
+		xmle.Elem
+		"""
+		result = self.__xml__(output='png', use_viz=False, figsize=figsize)
+		if filename is not None:
+			import base64
+			if result.attrib['src'][:22] != "data:image/png;base64,":
+				raise ValueError("problem decoding png:{}".format(result.attrib['src'][:22]))
+			with open(filename, "wb") as fh:
+				fh.write(base64.decodebytes(result.attrib['src'][22:].encode()))
+		return result
+
 	def partial_figure(self, including_nodes=None, source=None, *, n=None, n_at_level=3, n_expand=1):
 		"""
 		Generate a partial figure of the graph.
@@ -562,7 +657,7 @@ class NestingTree(TouchNotify,nx.DiGraph):
 			for i in tier:
 				next_tier.extend(self.successors(i))
 
-	def _get_simple_mu_and_alpha(self, model):
+	def _get_simple_mu_and_alpha(self, model, holdfast_invalidates=True):
 		alpha = numpy.zeros([len(self), len(self)], dtype=numpy.float64)
 		mu    = numpy.ones ([len(self),          ], dtype=numpy.float64)
 		muslots= numpy.full([len(self),          ], -1, dtype=numpy.int32)
@@ -571,7 +666,7 @@ class NestingTree(TouchNotify,nx.DiGraph):
 				alpha[parent, child] = 1
 			pname = self.nodes[childcode].get('parameter', None)
 			mu[child] = model.get_value(pname, default=1.0)
-			muslots[child] = model.get_slot_x(pname, True)
+			muslots[child] = model.get_slot_x(pname, holdfast_invalidates)
 
 		s = self.n_edges
 		up = numpy.zeros(s, dtype=numpy.int32)
@@ -598,7 +693,7 @@ class NestingTree(TouchNotify,nx.DiGraph):
 
 		return mu, muslots, alpha, up, dn, num, start, val
 
-def graph_to_figure(graph, **format):
+def graph_to_figure(graph, output_format='svg', **format):
 
 	try:
 		import pygraphviz as viz
@@ -663,18 +758,20 @@ def graph_to_figure(graph, **format):
 		up_nodes.add(i)
 	pyg_imgdata = BytesIO()
 	try:
-		G.draw(pyg_imgdata, format='svg', prog='dot')       # write postscript in k5.ps with neato layout
+		G.draw(pyg_imgdata, format=output_format, prog='dot')       # write postscript in k5.ps with neato layout
 	except ValueError as err:
 		if 'in path' in str(err):
 			import warnings
 			warnings.warn(str(err)+"; unable to draw nesting tree in report")
 			raise NotImplementedError()
-	import xml.etree.ElementTree as ET
-	ET.register_namespace("","http://www.w3.org/2000/svg")
-	ET.register_namespace("xlink","http://www.w3.org/1999/xlink")
-	result = ET.fromstring(pyg_imgdata.getvalue().decode())
-
 	from xmle import Elem
+	if output_format == 'svg':
+		import xml.etree.ElementTree as ET
+		ET.register_namespace("","http://www.w3.org/2000/svg")
+		ET.register_namespace("xlink","http://www.w3.org/1999/xlink")
+		result = ET.fromstring(pyg_imgdata.getvalue().decode())
+	else:
+		result = Elem('span', attrib={'style':'color:red'}, text=f"Unable to render output_format '{output_format}'")
 	x = Elem('div') << result
 	return x
 
